; #INDEX# =======================================================================================================================
; Title .........: OpenJson
; AutoIt Version : 3.3.14.5
; Description ...: Get data from a text string in JSON format in a simple way
; Author(s) .....: Diego Román (digrom)
; ===============================================================================================================================

; #CURRENT# =====================================================================================================================
;_OJParse
; ===============================================================================================================================

; #INTERNAL_USE_ONLY# ===========================================================================================================
;_Format
;_GetArray
;_GetObject
;_StringMidIndex
; ===============================================================================================================================

; #VARIABLES# ===================================================================================================================
Global $_sJson = ''
Global $_iPos = 1
Global $_iLen = 0
; ===============================================================================================================================

; #FUNCTION# ====================================================================================================================
; Name...........: _OJParse
; Description ...: Get a dictionary object that contains the data obtained from the text string in JSON format
; Syntax.........: _OJParse($_sText)
; Parameters ....: $_sText	    - Text string in JSON format
; Return values .: Success      - Dictionary object that contains the data obtained from the text string in JSON format
;                  Failure      - Null
; Author ........: Diego Román (digrom)
; Modified.......: Nov/27/2021
; Remarks .......: This function takes a text string in JSON format and returns the data in a dictionary object
; ===============================================================================================================================
Func _OJParse($_sText)
	$_sJson = _Format($_sText)
	$_iLen = StringLen($_sJson)
	$_iPos = 1
	Local $_sToken = StringLeft($_sJson, 1)
	If $_sToken = '[' Then
		Return _GetArray()
	ElseIf $_sToken = '{' Then
		Return _GetObject()
	EndIf
EndFunc   ;==>_OJParse

; #INTERNAL_USE_ONLY# ===========================================================================================================
; Name...........: _GetArray
; Description ...: Make a dictionary object created from an JSON array
; Syntax.........: _GetArray()
; Return values .: Success      - Dictionary object that contains the data obtained from an JSON array
;                  Failure      - Empty or incomplete dictionary object
; Author ........: Diego Román (digrom)
; Modified.......: Nov/27/2021
; Remarks .......: This function processes a JSON array and returns the data in a dictionary object
; ===============================================================================================================================
Func _GetArray()
	Local $_dArrayNode = ObjCreate("Scripting.Dictionary")
	Local $_iIndex = 0, $_sText = ''
	$_iPos += 1
	While StringMid($_sJson, $_iPos, 1) <> ']' And $_iPos <= $_iLen
		Local $_sToken = StringMid($_sJson, $_iPos, 1)
		If $_sToken = ',' Then
			$_iPos += 1
		ElseIf $_sToken = '[' Then
			$_iIndex += 1
			$_dArrayNode.Add($_iIndex, _GetArray())
		ElseIf $_sToken = '{' Then
			$_iIndex += 1
			$_dArrayNode.Add($_iIndex, _GetObject())
		ElseIf $_sToken = '"' Then
			$_sText = ''
			If StringMid($_sJson, $_iPos + 1, 1) = '"' Then
				$_iPos += 2
			Else
				Local $_iPos_end = StringInStr($_sJson, '"', 0, 1, $_iPos + 1)
				$_sText = _StringMidIndex($_sJson, $_iPos + 1, $_iPos_end - 1)
				$_iPos = $_iPos_end + 1
			EndIf
			$_iIndex += 1
			$_dArrayNode.Add($_iIndex, $_sText)
		Else
			$_sText = ''
			While StringMid($_sJson, $_iPos, 1) <> ',' And StringMid($_sJson, $_iPos, 1) <> ']' And $_iPos <= $_iLen
				$_sText &= StringMid($_sJson, $_iPos, 1)
				$_iPos += 1
			WEnd
			$_iIndex += 1
			$_dArrayNode.Add($_iIndex, $_sText)
		EndIf
	WEnd
	$_iPos += 1
	Return $_dArrayNode
EndFunc   ;==>_GetArray

; #INTERNAL_USE_ONLY# ===========================================================================================================
; Name...........: _GetObject
; Description ...: Make a dictionary object created from an JSON object
; Syntax.........: _GetObject()
; Return values .: Success      - Dictionary object that contains the data obtained from an JSON object
;                  Failure      - Empty or incomplete dictionary object
; Author ........: Diego Román (digrom)
; Modified.......: Nov/27/2021
; Remarks .......: This function processes a JSON object and returns the data in a dictionary object
; ===============================================================================================================================
Func _GetObject()
	Local $_dObjectNode = ObjCreate("Scripting.Dictionary")
	Local $_sKey = '', $_sText = ''
	$_iPos += 1
	While StringMid($_sJson, $_iPos, 1) <> '}' And $_iPos <= $_iLen
		Local $_sToken = StringMid($_sJson, $_iPos, 1)
		If $_sToken = ',' Then
			$_iPos += 1
		ElseIf $_sToken = '[' Then
			$_dObjectNode.Add($_sKey, _GetArray())
		ElseIf $_sToken = '{' Then
			$_dObjectNode.Add($_sKey, _GetObject())
		ElseIf $_sToken = '"' Then
			$_sText = ''
			If StringMid($_sJson, $_iPos + 1, 1) = '"' Then
				$_iPos += 2
			Else
				Local $_iPos_end = StringInStr($_sJson, '"', 0, 1, $_iPos + 1)
				$_sText = _StringMidIndex($_sJson, $_iPos + 1, $_iPos_end - 1)
				$_iPos = $_iPos_end + 1
			EndIf
			If StringMid($_sJson, $_iPos, 1) = ':' Then
				$_sKey = $_sText
				$_iPos += 1
			Else
				$_dObjectNode.Add($_sKey, $_sText)
			EndIf
		Else
			$_sText = ''
			While StringMid($_sJson, $_iPos, 1) <> ',' And StringMid($_sJson, $_iPos, 1) <> '}' And $_iPos <= $_iLen
				$_sText &= StringMid($_sJson, $_iPos, 1)
				$_iPos += 1
			WEnd
			$_dObjectNode.Add($_sKey, $_sText)
		EndIf
	WEnd
	$_iPos += 1
	Return $_dObjectNode
EndFunc   ;==>_GetObject

; #INTERNAL_USE_ONLY# ===========================================================================================================
; Name...........: _StringMidIndex
; Description ...: Get a substring of text located between two specified indexes
; Syntax.........: _StringMidIndex($_sText, $_i, $_j)
; Return values .: Substring located between two indices
; Author ........: Diego Román (digrom)
; Modified.......: Nov/27/2021
; ===============================================================================================================================
Func _StringMidIndex($_sText, $_i, $_j)
	Return StringMid($_sText, $_i, ($_j - $_i) + 1)
EndFunc   ;==>_StringMidIndex

; #INTERNAL_USE_ONLY# ===========================================================================================================
; Name...........: _Format
; Description ...: Remove all unnecessary tab characters, newlines, and whitespace
; Syntax.........: _Format($_sText)
; Return values .: A text string with no tab characters, line breaks, and unnecessary spaces
; Author ........: Diego Román (digrom)
; Modified.......: Nov/27/2021
; ===============================================================================================================================
Func _Format($_sText)
	Local $_iCom = 0, $_sChr = "", $_sCText = ""
	For $_i = 1 To StringLen($_sText)
		$_sChr = StringMid($_sText, $_i, 1)
		Switch $_sChr
			Case @CR, @LF, @TAB
				ContinueLoop
			Case '"'
				If $_i = 1 Then
					$_iCom += 1
				ElseIf StringMid($_sText, $_i - 1, 1) <> "\" Then
					$_iCom += 1
				EndIf
			Case " "
				If Mod($_iCom, 2) = 0 Then
					ContinueLoop
				EndIf
		EndSwitch
		$_sCText &= $_sChr
	Next
	Return $_sCText
EndFunc   ;==>_Format


